@@ -14,34 +14,36 @@ from quiffen.core.base import BaseModel, Field
 from quiffen.core.category import Category, add_categories_to_container
 from quiffen.core.class_type import Class
 from quiffen.core.investment import Investment
-from quiffen.core.transaction import Transaction
 from quiffen.core.security import Security
+from quiffen.core.transaction import Transaction
 
 try:
     import pandas as pd
+
     PANDAS_INSTALLED = True
 except ModuleNotFoundError:
     PANDAS_INSTALLED = False
 
 VALID_TRANSACTION_ACCOUNT_TYPES = [
-    '!type:cash',
-    '!type:bank',
-    '!type:ccard',
-    '!type:otha',
-    '!type:othl',
-    '!type:invoice'
+    "!type:cash",
+    "!type:bank",
+    "!type:ccard",
+    "!type:otha",
+    "!type:othl",
+    "!type:invoice",
 ]
 
 
 class QifDataType(str, Enum):
     """An Enum representing the different types of data that can be found in a
     Qif object."""
-    TRANSACTIONS = 'transactions'
-    INVESTMENTS = 'investments'
-    CLASSES = 'classes'
-    CATEGORIES = 'categories'
-    ACCOUNTS = 'accounts'
-    SECURITIES = 'securities'
+
+    TRANSACTIONS = "transactions"
+    INVESTMENTS = "investments"
+    CLASSES = "classes"
+    CATEGORIES = "categories"
+    ACCOUNTS = "accounts"
+    SECURITIES = "securities"
 
 
 class ParserException(Exception):
@@ -63,48 +65,46 @@ class Qif(BaseModel):
     classes : dict, default=None
         A dict of classes in the form {'Class Name': class_object}.
     """
+
     accounts: Dict[str, Account] = {}
     categories: Dict[str, Category] = {}
     classes: Dict[str, Class] = {}
     securities: Dict[str, Security] = {}
 
-    __CUSTOM_FIELDS: List[Field] = []
+    __CUSTOM_FIELDS: List[Field] = []  # type: ignore
 
     def __str__(self) -> str:
-        accounts_str = '\n'.join(str(acc) for acc in self.accounts.values())
-        categories_str = '\n'.join(str(cat) for cat in self.categories.values())
-        classes_str = '\n'.join(str(cls) for cls in self.classes.values())
-        securities_str = '\n'.join(str(sec) for sec in self.securities.values())
+        accounts_str = "\n".join(str(acc) for acc in self.accounts.values())
+        categories_str = "\n".join(str(cat) for cat in self.categories.values())
+        classes_str = "\n".join(str(cls) for cls in self.classes.values())
+        securities_str = "\n".join(str(sec) for sec in self.securities.values())
 
         if not (accounts_str or categories_str or classes_str):
-            return 'Empty Qif object'
+            return "Empty Qif object"
 
-        return_str = 'QIF\n===\n\n'
+        return_str = "QIF\n===\n\n"
 
         if accounts_str:
-            return_str += f'Accounts\n--------\n\n{accounts_str}\n\n'
+            return_str += f"Accounts\n--------\n\n{accounts_str}\n\n"
         if categories_str:
-            return_str += f'Categories\n----------\n\n{categories_str}\n\n'
+            return_str += f"Categories\n----------\n\n{categories_str}\n\n"
         if classes_str:
-            return_str += f'Classes\n-------\n\n{classes_str}\n\n'
+            return_str += f"Classes\n-------\n\n{classes_str}\n\n"
         if securities_str:
-            return_str += f'Securities\n----------\n\n{securities_str}\n\n'
+            return_str += f"Securities\n----------\n\n{securities_str}\n\n"
 
         return return_str
 
     @classmethod
     def from_list(cls, lst: List[str]) -> Qif:
         raise NotImplementedError(
-            'This method is not implemented for Qif objects. Use Qif.parse to '
-            'parse a QIF file.'
+            "This method is not implemented for Qif objects. Use Qif.parse to "
+            "parse a QIF file."
         )
 
     @classmethod
     def parse(
-        cls,
-        path: Union[FilePath, str],
-        separator: str = '\n',
-        day_first: bool = False
+        cls, path: Union[FilePath, str], separator: str = "\n", day_first: bool = False
     ) -> Qif:
         """Return a class instance from a QIF file.
 
@@ -124,16 +124,16 @@ class Qif(BaseModel):
             A Qif object containing all the data in the QIF file.
         """
         path = Path(path)
-        if path.suffix.lower() != '.qif':
-            raise ParserException('The file must be a QIF file.')
+        if path.suffix.lower() != ".qif":
+            raise ParserException("The file must be a QIF file.")
 
         if not path.exists():
-            raise ParserException('The file does not exist.')
+            raise ParserException("The file does not exist.")
 
-        data = path.read_text(encoding='utf-8').strip().strip('\n')
+        data = path.read_text(encoding="utf-8").strip().strip("\n")
 
         if not data:
-            raise ParserException('The file is empty.')
+            raise ParserException("The file is empty.")
 
         accounts: Dict[str, Account] = {}
         last_account = None
@@ -141,7 +141,7 @@ class Qif(BaseModel):
         classes: Dict[str, Class] = {}
         securities: Dict[str, Security] = {}
 
-        sections = data.split('^')
+        sections = data.split("^")
         last_header = None
         line_number = 1
 
@@ -156,7 +156,7 @@ class Qif(BaseModel):
             # Allow for comments and blank lines at the top of sections
             for i, line in enumerate(section_lines):
                 header_line = line
-                if line.strip() and line[0] != '#':
+                if line.strip() and line[0] != "#":
                     line_number += i
                     section_lines = section_lines[i:]
                     break
@@ -164,19 +164,19 @@ class Qif(BaseModel):
                 # Empty section
                 continue
 
-            if header_line[0] != '!':
+            if header_line[0] != "!":
                 if not last_header:
                     raise ParserException(
-                        f'Line {line_number}: '
-                        f'No header found before transactions.'
+                        f"Line {line_number}: " f"No header found before transactions."
                     )
                 header_line = last_header
 
             last_header = header_line
 
             sanitised_section_lines = [
-                line for line in section_lines
-                if line.strip() and line.strip()[0] != '!'
+                line
+                for line in section_lines
+                if line.strip() and line.strip()[0] != "!"
             ]
 
             if not sanitised_section_lines:
@@ -184,58 +184,70 @@ class Qif(BaseModel):
 
             # Check for new categories and accounts first, otherwise it's a
             # transaction so a default account is created
-            if '!Type:Cat' in header_line:
+            if "!Type:Cat" in header_line:
                 # Section contains category information
                 new_category = Category.from_list(sanitised_section_lines)
-                categories = add_categories_to_container(
+                categories = add_categories_to_container(  # type: ignore
                     new_category,
                     categories,
                 )
-            elif '!Type:Class' in header_line:
+            elif "!Type:Class" in header_line:
                 new_class = Class.from_list(sanitised_section_lines)
                 if new_class.name in classes:
                     classes[new_class.name].merge(new_class)
                 else:
                     classes[new_class.name] = new_class
-            elif '!Account' in header_line:
+            elif "!Account" in header_line:
                 new_account = Account.from_list(sanitised_section_lines)
                 if new_account.name in accounts:
                     accounts[new_account.name].merge(new_account)
                 else:
                     accounts[new_account.name] = new_account
                 last_account = new_account.name
-            elif '!Type:Invst' in header_line:
+            elif "!Type:Invst" in header_line:
                 # Investment
                 new_investment = Investment.from_list(
                     sanitised_section_lines,
                     day_first=day_first,
                     line_number=line_number,
                 )
+
+                if last_account is None:
+                    raise ParserException(
+                        f"Line {line_number}: "
+                        "No account found before investment. "
+                        "This should not happen."
+                    )
+
                 accounts[last_account].add_transaction(
                     new_investment,
-                    AccountType('Invst'),
+                    AccountType("Invst"),
                 )
-            elif '!Type:Security' in header_line:
+            elif "!Type:Security" in header_line:
                 # Security
                 new_security = Security.from_list(
                     sanitised_section_lines,
                     line_number=line_number,
                 )
+                if new_security.symbol is None:
+                    raise ParserException(
+                        f"Line {line_number}: " f"No symbol found for security."
+                    )
                 securities[new_security.symbol] = new_security
-            elif '!Type' in header_line and not accounts:
+            elif "!Type" in header_line and not accounts:
                 # Accounts is empty and there's a transaction, so create default
                 # account to put transactions in
                 default_account = Account(
-                    name='Quiffen Default Account',
+                    name="Quiffen Default Account",
                     desc=(
-                        'The default account created by Quiffen when no other '
-                        'accounts were present'
-                    )
+                        "The default account created by Quiffen when no other "
+                        "accounts were present"
+                    ),
                 )
                 accounts[default_account.name] = default_account
                 last_account = default_account.name
 
-            if header_line.lower().replace(' ', '') in (
+            if header_line.lower().replace(" ", "") in (
                 VALID_TRANSACTION_ACCOUNT_TYPES
             ):
                 # Other transaction type
@@ -244,9 +256,17 @@ class Qif(BaseModel):
                     day_first=day_first,
                     line_number=line_number,
                 )
+
+                if last_account is None:
+                    raise ParserException(
+                        f"Line {line_number}: "
+                        "No account found before transactions. "
+                        "This should not happen."
+                    )
+
                 accounts[last_account].add_transaction(
                     new_transaction,
-                    AccountType(header_line.split(':')[1]),
+                    AccountType(header_line.split(":")[1]),
                 )
 
                 if new_transaction.category:
@@ -262,7 +282,7 @@ class Qif(BaseModel):
                     else:
                         classes[class_name] = new_class
 
-            line_number += len(section.split('\n'))
+            line_number += len(section.split("\n"))
 
         return cls(
             accounts=accounts,
@@ -289,7 +309,7 @@ class Qif(BaseModel):
 
     def add_category(self, new_category: Category) -> None:
         """Add a new category to the Qif object"""
-        self.categories = add_categories_to_container(
+        self.categories = add_categories_to_container(  # type: ignore
             new_category,
             self.categories,
         )
@@ -308,7 +328,7 @@ class Qif(BaseModel):
             ) from e
 
         if keep_children:
-            print('Keeping children')
+            print("Keeping children")
             for child in category.children:
                 child.set_parent(None)
                 self.add_category(child)
@@ -333,6 +353,11 @@ class Qif(BaseModel):
 
     def add_security(self, new_security: Security) -> None:
         """Add a new security to the Qif object"""
+        if not new_security.symbol:
+            raise ValueError(
+                "Cannot add a security without a symbol to the Qif object."
+            )
+
         if new_security.symbol in self.securities:
             self.securities[new_security.symbol].merge(new_security)
         else:
@@ -344,46 +369,45 @@ class Qif(BaseModel):
             return self.securities.pop(security_symbol)
         except KeyError as e:
             raise KeyError(
-                f'Security "{security_symbol}" does not exist in this Qif '
-                f'object.'
+                f'Security "{security_symbol}" does not exist in this Qif ' f"object."
             ) from e
 
     def to_qif(
         self,
-        path: Union[FilePath, str, None] = None,
-        date_format: str = '%Y-%m-%d',
+        path: Optional[Union[FilePath, str, None]] = None,
+        date_format: str = "%Y-%m-%d",
     ) -> str:
         """Convert the Qif object to a QIF file"""
-        qif = ''
+        qif = ""
 
         if self.categories:
-            qif += '^\n'.join(
-                category.to_qif()
-                for category in self.categories.values()
-            ) + '^\n'
+            qif += (
+                "^\n".join(category.to_qif() for category in self.categories.values())
+                + "^\n"
+            )
 
         if self.classes:
-            qif += '^\n'.join(
-                cls.to_qif()
-                for cls in self.classes.values()
-            ) + '^\n'
+            qif += "^\n".join(cls.to_qif() for cls in self.classes.values()) + "^\n"
 
         if self.accounts:
-            qif += '^\n'.join(
-                account.to_qif(date_format=date_format, classes=self.classes)
-                for account in self.accounts.values()
-            ) + '^\n'
+            qif += (
+                "^\n".join(
+                    account.to_qif(date_format=date_format, classes=self.classes)
+                    for account in self.accounts.values()
+                )
+                + "^\n"
+            )
 
         if path:
-            Path(path).write_text(qif, encoding='utf-8')
+            Path(path).write_text(qif, encoding="utf-8")
 
         return qif
 
     def _get_data_dicts(
         self,
         data_type: QifDataType = QifDataType.TRANSACTIONS,
-        date_format: Optional[str] = '%Y-%m-%d',
-        ignore: List[str] = None,
+        date_format: Optional[str] = "%Y-%m-%d",
+        ignore: Optional[List[str]] = None,
     ) -> List[Dict[str, Any]]:
         """Converts specified data from the Qif object to a list of dicts"""
         if ignore is None:
@@ -409,36 +433,33 @@ class Qif(BaseModel):
                     ]
         elif data_type == QifDataType.ACCOUNTS:
             data_dicts = [
-                account.to_dict(ignore=ignore)
-                for account in self.accounts.values()
+                account.to_dict(ignore=ignore) for account in self.accounts.values()
             ]
         elif data_type == QifDataType.CATEGORIES:
             data_dicts = [
-                category.to_dict(ignore=ignore)
-                for category in self.categories.values()
+                category.to_dict(ignore=ignore) for category in self.categories.values()
             ]
         elif data_type == QifDataType.CLASSES:
             data_dicts = [
-                class_.to_dict(ignore=ignore)
-                for class_ in self.classes.values()
+                class_.to_dict(ignore=ignore) for class_ in self.classes.values()
             ]
         elif data_type == QifDataType.SECURITIES:
             data_dicts = [
-                security.to_dict(ignore=ignore)
-                for security in self.securities.values()
+                security.to_dict(ignore=ignore) for security in self.securities.values()
             ]
         else:
             raise ValueError(
-                f'Invalid data_type: {data_type}. Must be one of '
-                f'{list(QifDataType)}'
+                f"Invalid data_type: {data_type}. Must be one of "
+                f"{list(QifDataType)}"
             )
 
         # Format and hide private fields
-        return [
+        return [  # type: ignore
             utils.apply_csv_formatting_to_container(
                 {
-                    k: v for k, v in data_dict.items()
-                    if not k.startswith('_') and k not in ignore
+                    k: v
+                    for k, v in data_dict.items()
+                    if not k.startswith("_") and k not in ignore
                 },
                 date_format=date_format,
             )
@@ -447,11 +468,11 @@ class Qif(BaseModel):
 
     def to_csv(
         self,
-        path: Union[FilePath, str, None] = None,
+        path: Optional[Union[FilePath, str, None]] = None,
         data_type: QifDataType = QifDataType.TRANSACTIONS,
-        date_format: str = '%Y-%m-%d',
-        ignore: List[str] = None,
-        delimiter: str = ',',
+        date_format: str = "%Y-%m-%d",
+        ignore: Optional[List[str]] = None,
+        delimiter: str = ",",
         quote_character: str = '"',
     ) -> str:
         """Convert part of the Qif object to a CSV file. The data_type
@@ -495,8 +516,8 @@ class Qif(BaseModel):
         writer = csv.DictWriter(
             output,
             fieldnames=headers,
-            extrasaction='ignore',  # Ignore extra fields (e.g. private fields)
-            dialect='unix',  # Use Unix line endings
+            extrasaction="ignore",  # Ignore extra fields (e.g. private fields)
+            dialect="unix",  # Use Unix line endings
             delimiter=delimiter,
             quotechar=quote_character,
         )
@@ -507,14 +528,14 @@ class Qif(BaseModel):
         return_value = output.getvalue()
 
         if path:
-            Path(path).write_text(return_value, encoding='utf-8')
+            Path(path).write_text(return_value, encoding="utf-8")
 
         return return_value
 
     def to_dataframe(
         self,
         data_type: QifDataType = QifDataType.TRANSACTIONS,
-        ignore: List[str] = None,
+        ignore: Optional[List[str]] = None,
     ) -> pd.DataFrame:
         """Convert part of the Qif object to a Pandas DataFrame. The
         data_type parameter can be used to specify which part of the Qif
@@ -537,7 +558,7 @@ class Qif(BaseModel):
         """
         if not PANDAS_INSTALLED:
             raise ModuleNotFoundError(
-                'The pandas module must be installed to use this method'
+                "The pandas module must be installed to use this method"
             )
 
         if ignore is None:
